@@ -11,18 +11,14 @@
 # Copyright (c) 2002-2020, RDFLib Team and is distributed under a similar 3-clause BSD
 # License; see this file: https://github.com/RDFLib/rdflib/blob/master/LICENSE
 
-from xmptools.xmptools import makeFileURI, isContainerItemPredicate, makeContainerItemPredicate
-from xmptools.xmptools import getContainerStatements, cbd, identity, makeFileURI, makeFilePath
-from xmptools.xmptools import XMPMetadata
+from xmptools.xmptools import makeFileURI, makeFileURI, makeFilePath, XMPMetadata
 from xmptools.xmptools import DC, DCT, XMP, EXIF, CRS, PHOTOSHOP
 from xmptools.xmptools import JPEG_EXTENSIONS, TIFF_EXTENSIONS, RAW_EXTENSIONS, DNG_EXTENSIONS
 from xmptools.xmptools import XMP_EXTENSIONS
 from xmptools.xmptools import FileTypeError, XMPParser, XMPSerializer
 
 __all__ = [
-    'makeFileURI', 'isContainerItemPredicate', 'makeContainerItemPredicate',
-    'getContainerStatements', 'cbd', 'identity', 'makeFileURI', 'makeFilePath',
-    'XMPMetadata',
+    'makeFileURI', 'makeFileURI', 'makeFilePath', 'XMPMetadata',
     'DC', 'DCT', 'XMP', 'EXIF', 'CRS', 'PHOTOSHOP',
     'JPEG_EXTENSIONS', 'TIFF_EXTENSIONS', 'RAW_EXTENSIONS', 'DNG_EXTENSIONS',
     'XMP_EXTENSIONS',
