@@ -10,8 +10,9 @@
 # modified because RDFLib did not provide suitable extension points. That code is
 # Copyright (c) 2002-2020, RDFLib Team and is distributed under a similar 3-clause BSD
 # License; see this file: https://github.com/RDFLib/rdflib/blob/master/LICENSE
-import rdflib
-from rdflib import RDF, URIRef, BNode, Literal, Namespace, Graph, plugin
+
+import gzip
+from rdflib import RDF, URIRef, Literal, Namespace, Graph, plugin
 from rdflib.exceptions import Error
 from rdflib.parser import Parser
 from rdflib.serializer import Serializer
@@ -24,6 +25,7 @@ from datetime import date, datetime, tzinfo, timedelta
 import os.path
 import pathlib
 from urllib.parse import urlparse, unquote_plus
+import rdfhelpers
 
 TOOLNAME = "So Many Aircraft xmptools 0.4"
 XMP_NS = "adobe:ns:meta/"
@@ -33,27 +35,12 @@ XMP_TAG_CLOSE = '</x:{0}>'.format(XMP_NAME).encode('utf-8')
 XMP_TAG_OPEN_FULL = \
     '<x:{0} xmlns:x="{1}" x:xmptk="{2}">\n'.format(XMP_NAME, XMP_NS, TOOLNAME).encode('utf-8')
 
-
 class FileTypeError(Error):
     def __init__(self, message, path, original=None):
         super().__init__(message.format(path, original))
         self.path = path
         self.original = original
 
-
-LI_MATCH_PATTERN = re.compile(str(RDF) + "_([0-9]+)")
-LI_CREATE_PATTERN = str(RDF) + "_{0}"
-
-
-def isContainerItemPredicate(uri):
-    match = LI_MATCH_PATTERN.match(uri)
-    return int(match.group(1)) if match else None
-
-
-def makeContainerItemPredicate(index):
-    return LI_CREATE_PATTERN.format(index)
-
-
 class XMPHandler(RDFXMLHandler):
     def reset(self):
         super().reset()
@@ -75,7 +62,6 @@ class XMPHandler(RDFXMLHandler):
             self.error("Repeated node-elements: %s" % "".join(name))
         self.parent.object = self.current.subject
 
-
 class XMPParser(Parser):
     def __init__(self):
         self._parser = None
@@ -98,7 +84,6 @@ class XMPParser(Parser):
         except SAXParseException as e:
             raise FileTypeError('Possibly not XMP because "{1}"', None, original=e)
 
-
 class XMPSerializer(PrettyXMLSerializer):
     # We must subclass PrettyXMLSerializer, because the Adobe XMP toolkit expects
     # blank nodes to be serialized "in line" and not as separate Descriptions. The
@@ -130,8 +115,8 @@ class XMPSerializer(PrettyXMLSerializer):
 
     def predicate(self, pred, obj, depth=1):
         # Replace actual container item predicates with <rdf:li> as per the XMP spec
-        super().predicate(self.RDFLI if isContainerItemPredicate(pred) else pred, obj, depth)
-
+        super().predicate(self.RDFLI if rdfhelpers.isContainerItemPredicate(pred) else pred,
+                          obj, depth)
 
 plugin.register("xmp", Parser, "xmptools", "XMPParser")
 plugin.register("xmp", Serializer, "xmptools", "XMPSerializer")
@@ -143,23 +128,12 @@ DC = Namespace("http://purl.org/dc/elements/1.1/")
 DCT = Namespace("http://purl.org/dc/terms/")
 PHOTOSHOP = Namespace("http://ns.adobe.com/photoshop/1.0/")
 
-
-class MoreThanOneValue(Error):
-    pass
-
-
 class DataNotFound(Error):
     pass
 
-
-def URI(u):
-    return u if isinstance(u, URIRef) or u is None else URIRef(u)
-
-
 def makeFileURI(path):
     return URIRef(pathlib.Path(os.path.abspath(path)).as_uri())
 
-
 def makeFilePath(uri, scheme="file"):
     u = urlparse(uri)
     if u.scheme == scheme:
@@ -167,20 +141,6 @@ def makeFilePath(uri, scheme="file"):
     else:
         raise ValueError("URI does not have '{}' scheme: {}".format(scheme, str(uri)))
 
-
-def getContainerStatements(graph, source, predicate):
-    containers = list(graph.objects(URI(source), predicate))
-    n = len(containers)
-    if n == 1:
-        return sorted([statement for statement in graph.triples((containers[0], None, None))
-                       if isContainerItemPredicate(statement[1])],
-                      key=lambda tr: tr[1])
-    elif n == 0:
-        return None
-    else:
-        raise MoreThanOneValue("Expected only one value for {0}".format(predicate))
-
-
 def adjustTriple(triple, node_map, check_predicates=False):
     s, p, o = triple
     for node in node_map:
@@ -192,7 +152,6 @@ def adjustTriple(triple, node_map, check_predicates=False):
             o = node_map[node]
     return s, p, o
 
-
 def adjustNodes(node_map, source_graph, destination_graph=None, check_predicates=False):
     if destination_graph is None:
         destination_graph = type(source_graph)()
@@ -203,7 +162,6 @@ def adjustNodes(node_map, source_graph, destination_graph=None, check_predicates
         destination_graph.sourceIsXMP = False
     return destination_graph
 
-
 def adjustNodesInPlace(node_map, graph, check_predicates=False):
     for triple in list(graph.triples((None, None, None))):
         new_triple = adjustTriple(triple, node_map, check_predicates=check_predicates)
@@ -214,24 +172,6 @@ def adjustNodesInPlace(node_map, graph, check_predicates=False):
         graph.url = node_map[graph.url]
     return graph
 
-
-def identity(x):
-    return x
-
-
-def cbd(source, target, resource):
-    # Does not support reified statements (yet)
-    subjects = list([resource])
-    while subjects:
-        subject = subjects.pop()
-        for triple in source.triples((subject, None, None)):
-            target.add(triple)
-            leaf = triple[2]
-            if isinstance(leaf, rdflib.BNode):
-                subjects.append(leaf)
-    return target
-
-
 class iso8601:
     patterns =\
         {'iso8601': (("(?P<y>[0-9]{4})-(?P<mo>[0-9]{2})-(?P<d>[0-9]{2})"
@@ -275,7 +215,6 @@ class iso8601:
                                         timezone())
         return None
 
-
 class timezone(tzinfo):
     def __init__(self, oh=0, om=0, negative=False):
         m = (int(oh) if oh else 0) * 60 + (int(om) if om else 0)
@@ -287,14 +226,12 @@ class timezone(tzinfo):
     def dst(self, dt):
         return None
 
-
 JPEG_EXTENSIONS = [".jpg", ".jpeg", ".JPG", ".JPEG"]
 TIFF_EXTENSIONS = [".tif", ".tiff", ".TIF", ".TIFF"]
 DNG_EXTENSIONS = [".dng", ".DNG"]
 RAW_EXTENSIONS = [".cr2", ".CR2", ".cr3", ".CR3"]  # obviously wholly incomplete still
 XMP_EXTENSIONS = [".xmp", ".XMP"]
 
-
 class XMPMetadata(Graph):
     iso8601 = iso8601()
 
@@ -305,7 +242,7 @@ class XMPMetadata(Graph):
             self.sourceIsXMP = False  # signals that content didn't come from an XMP sidecar file
             self.segment_count = 0
         else:
-            self.url = URI(url)
+            self.url = rdfhelpers.URI(url)
             self.sourceIsXMP = True
             self.segment_count = 1
         super().__init__(identifier=self.url)
@@ -424,35 +361,14 @@ class XMPMetadata(Graph):
             return None, None
 
     def getContainerItems(self, predicate):
-        statements = getContainerStatements(self, self.url, predicate)
-        return [statement[2] for statement in statements] if statements else None
+        return rdfhelpers.getContainerItems(self, self.url, predicate)
 
     def setContainerItems(self, predicate, values, newtype=RDF.Seq):
-        # Having to write code like this is a clear indication that triples are the wrong
-        # abstraction for graphs, way too low level. Just sayin'.
-        if values and len(values) > 0:
-            statements = getContainerStatements(self, self.url, predicate)
-            if statements:
-                container = statements[0][0]
-                for statement in statements:
-                    self.remove(statement)
-            else:
-                container = BNode()
-                self.add((self.url, predicate, container))
-                self.add((container, RDF.type, newtype))
-            i = 1
-            for value in values:
-                self.add((container, URIRef(makeContainerItemPredicate(i)), value))
-                i += 1
-        else:
-            statement = self.getOneStatement(predicate)
-            container = statement[2]
-            self.remove((self.url, predicate, container))
-            self.remove((container, None, None))
+        rdfhelpers.setContainerItems(self, self.url, predicate, values, newtype=newtype)
 
     def container2repeated(self, predicate,
-                           new_predicate=None, value_mapper=identity, remove_predicate=False,
-                           target_graph=None):
+                           new_predicate=None, value_mapper=rdfhelpers.identity,
+                           remove_predicate=False, target_graph=None):
         # We understand that this messes with ordering
         if new_predicate is None:
             new_predicate = predicate
@@ -467,7 +383,7 @@ class XMPMetadata(Graph):
         return target_graph
 
     def repeated2container(self, predicate,
-                           new_predicate=None, newtype=RDF.Seq, value_mapper=identity,
+                           new_predicate=None, newtype=RDF.Seq, value_mapper=rdfhelpers.identity,
                            remove_predicate=False, source_graph=None):
         # We understand that this messes with ordering
         if new_predicate is None:
@@ -482,15 +398,11 @@ class XMPMetadata(Graph):
             self.setContainerItems(new_predicate, [value_mapper(o) for _, _, o in statements],
                                    newtype=newtype)
 
-    def adjustImageURI(self, new_extension=None, destination_graph=None, uri_mapper=None):
-        # Logic:
-        #   1. If no new extension specified, use crs:RawFileName value for the new extension
-        #      - if no RawFileName found, signal an error
-        #   2. Otherwise, replace old extension with the new extension provided
-        original = self.getValue(CRS.RawFileName)
+    def findOriginalImageURI(self, new_extension=None):
+        original = str(rdfhelpers.getvalue(self, self.url, CRS.RawFileName))
         if new_extension is None:
             if original is None:
-                raise FileTypeError("No new extension specified for {0}", self.url)
+                raise ValueError("No new extension specified for {0}", self.url)
             else:
                 url = urlparse(str(self.url))
                 path, ext = os.path.splitext(url.path)
@@ -499,6 +411,14 @@ class XMPMetadata(Graph):
         else:
             base, _ = os.path.splitext(str(self.url))
             new_uri = base + new_extension
+        return new_uri
+
+    def adjustImageURI(self, new_extension=None, destination_graph=None, uri_mapper=None):
+        # Logic:
+        #   1. If no new extension specified, use crs:RawFileName value for the new extension
+        #      - if no RawFileName found, signal an error
+        #   2. Otherwise, replace old extension with the new extension provided
+        new_uri = self.findOriginalImageURI(new_extension=new_extension)
         if uri_mapper:
             new_uri = uri_mapper(new_uri)
         node_map = {self.url: URIRef(new_uri)}
@@ -507,6 +427,11 @@ class XMPMetadata(Graph):
         else:
             return adjustNodes(node_map, self, destination_graph=destination_graph)
 
-    def cbd(self, resource=None, target=None):
+    def cbd(self, resource=None, target=None, context=None):
         # Does not support reified statements (yet)
-        return cbd(self, target or Graph(), resource or self.url)
+        return rdfhelpers.cbd(self, target or Graph(), resource or self.url, context=context)
+
+    def archive(self, pathname):
+        _, ext = os.path.splitext(pathname)
+        with gzip.open((pathname + ".xmp.gz") if ext == '' else (pathname + ".gz"), "wb") as f:
+            self.write(f)
